# $Id: StochasticDAGGenerator.py,v 1.3 2006/01/31 20:37:30 nvf Exp $

#
# Preamble.
#

from glue import segments, lal
import onasys.DAGGenerator
from onasys import S5DAGGeneratorA
import logging
import popen2
import os, sys

#
# A DAG generator plug-in for making a single DAG that spans multiple
# segments.
#
# How to use this.  This DAG generator plug-in has a [daggenerator] section
# in the .ini file whose contents are identical to the [daggenerator]
# section used by the S5DAGGeneratorA plug-in.
#
# How this works.  This is based off of S5DAGGeneratorA.  The Config class
# for this plug-in is simply a copy of that plug-in's Config class (thus
# the .ini file is the same).  That plug-in defines a
# DAGGeneratorParameters class describing a DAG for a single segment
# (actually, just the start and stop times for the segment).  We define a
# DAGGeneratorParameters class for *this* plug-in as a list of
# DAGGeneratorParameters objects from the S5DAGGeneratorA plug-in.  We call
# that plug-in to generate its list, and then stick the entire list into
# one of our DAGGeneratorParameters objects.  We return that object as the
# single object in our list of DAGs returned to the daemon.
#

class DAGGeneratorConfig(S5DAGGeneratorA.DAGGeneratorConfig):
	pass

class DAGGeneratorParameters(onasys.DAGGenerator.DAGGeneratorParameters):
	def __init__(self, list):
		self.list = list
		self.extent = segments.segmentlist([dag.dataseg for dag in list]).extent()
                # list = list of segpairss; segpair = (dataseg, analseg)
                # want analysis boundaries
                self.analsegs = segments.segmentlist([dag.analseg for dag in list]).coalesce()

	def __repr__(self):
		return repr(self.list)

def DAGGeneratorParameterList(clock, config, available, used):
	# Ask S5DAGGeneratorA to make a list of DAGs.
	list = S5DAGGeneratorA.DAGGeneratorParameterList(clock, config, available, used)

	# Put that list into our DAGGeneratorParameters object and return
	# a list containing that one object back to the daemon.  If empty,
	# return an empty list rather than a list containing only an empty list.
	if list == []:
		return []
	return [DAGGeneratorParameters(list)]

class DAGGenerator(onasys.DAGGenerator.DAGGenerator):
	def dag_name(self):
		return self.tag + "-" + str(int(self.params.extent[0])) + "-" + str(int(self.params.extent.duration())) + ".dag"

        def generate_command(self):
                # Remember to enclose list in quotes for shell-safety
                command = " ".join([\
                          self.config.data_dir+"/"+self.config.generator,\
                          "-f",self.dag_name(),\
                          "-t",self.config.data_dir,\
                          "-a","\""+str(self.params.analsegs)+"\"",\
                          "-d","\""+str(self.params)+"\""\
                          ])
                if self.config.extra_args:
                        command += " " + self.config.extra_args
                return command

	def generate(self):
                logging.info(self.generate_command())
                child = popen2.Popen3(self.generate_command(), True)
                for line in child.childerr:
                        logging.warning(line.rstrip())
                newlyused = segments.segmentlist()
                try:
                        for line in child.fromchild:
                                newlyused |= segments.segmentlist([segments.segment(*map(lal.LIGOTimeGPS, line.split()))])
                except Exception, e:
                        raise onasys.DAGGenerator.DAGGeneratorError, "error parsing generator output \"" + line.rstrip() + "\": " + str(e)
                if len(newlyused) > 1:
                        logging.warning("generator script returned more than one analysis segment for removal")
                if len(newlyused) > 0:
                        if newlyused.extent() not in self.params.analsegs:
                                logging.warning("analysis segment to remove exceeds bounds of " + str(self.params.analsegs))
                else:
                        logging.warning("generator script returned no analysis segment for removal!")
                status = child.wait()
                if not os.WIFEXITED(status) or os.WEXITSTATUS(status):
                        raise onasys.DAGGenerator.DAGGeneratorError, "generator script exit code = " + str(os.WEXITSTATUS(status))
                return {"analysis": newlyused}
